package io.tswf.minecraft.metamorphosis.core.exception

import groovy.transform.CompileStatic
import io.tswf.minecraft.metamorphosis.core.model.FileHooksInfo
import io.tswf.minecraft.metamorphosis.core.reading.IFileHooksReadingProvider

@CompileStatic
class FileHooksReadingProviderException extends RuntimeException {
    IFileHooksReadingProvider readingProvider
    FileHooksInfo hooksInfo

    FileHooksReadingProviderException(IFileHooksReadingProvider readingProvider, FileHooksInfo hooksInfo) {
        super(getExceptionMessage(readingProvider, hooksInfo))
        this.readingProvider = readingProvider
        this.hooksInfo = hooksInfo
    }

    FileHooksReadingProviderException(String message, IFileHooksReadingProvider readingProvider, FileHooksInfo hooksInfo) {
        super(getExceptionMessage(readingProvider, hooksInfo, message))
        this.readingProvider = readingProvider
        this.hooksInfo = hooksInfo
    }

    FileHooksReadingProviderException(String message, Throwable cause, IFileHooksReadingProvider readingProvider, FileHooksInfo hooksInfo) {
        super(getExceptionMessage(readingProvider, hooksInfo, message), cause)
        this.readingProvider = readingProvider
        this.hooksInfo = hooksInfo
    }

    FileHooksReadingProviderException(Throwable cause, IFileHooksReadingProvider readingProvider, FileHooksInfo hooksInfo) {
        super(getExceptionMessage(readingProvider, hooksInfo), cause)
        this.readingProvider = readingProvider
        this.hooksInfo = hooksInfo
    }
    
    static String getExceptionMessage(IFileHooksReadingProvider readingProvider, FileHooksInfo hooksInfo, String message = "") {
        return "Exception occurred while trying to read hooks info from file '${hooksInfo?.file?.absoluteCanonicalFile?.path}' by '${readingProvider}': ${message}"
    }
}
