package io.tswf.minecraft.metamorphosis.core.reading

import groovy.transform.CompileStatic
import io.tswf.minecraft.metamorphosis.core.exception.FileHooksReadingException
import io.tswf.minecraft.metamorphosis.core.exception.FileHooksReadingProviderException
import io.tswf.minecraft.metamorphosis.core.model.FileHooksInfo

import java.util.stream.Stream
import java.util.zip.ZipFile

@CompileStatic
class FileHooksReader {

    @Lazy
    static List<IFileHooksReadingProvider> hooksReadingProviders = ServiceLoader.load(IFileHooksReadingProvider)
        .toList()

    static FileHooksInfo readFileHooks(File file) {
        try {
            def result = new FileHooksInfo(file: file)
            def hooksInfoReadingContext = new FileHooksReadingContext(
                    hooksInfo: result
            )

            for (def hooksReadingProvider : hooksReadingProviders) {
                try {
                    hooksReadingProvider.handleFileHooksReading(hooksInfoReadingContext)
                } catch (Throwable e) {
                    throw new FileHooksReadingProviderException(e, hooksReadingProvider, result)
                }
            }

            file.asFileTree().doThenClose { fileTree ->
                Stream.of(result.server, result.client)
                        .flatMap { it.transmutations.stream().joinWith(it.expansions.stream()).joinWith(it.views.stream()) }
                        .forEach {
                            def zipResource = fileTree.getEntry(it.replace(".", "/").concat(".class"))
                            def bytes = zipResource.getInputStream().getBytes()
                            result.client.classNameToBytes[it] = bytes
                            result.server.classNameToBytes[it] = bytes
                        }
            }

            return result
        } catch (Throwable e) {
            throw new FileHooksReadingException(e, file)
        }
    }
}
