package io.tswf.minecraft.metamorphosis.core.reading.reader

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.groovy.bettergroovy.io.tree.IFileTree
import io.tswf.groovy.bettergroovy.io.tree.IFileTreeEntry
import io.tswf.minecraft.metamorphosis.core.reading.FileHooksReadingContext
import io.tswf.minecraft.metamorphosis.core.reading.IFileHooksReadingProvider

@Slf4j
@CompileStatic
abstract class AbstractMetaInfInfoReader implements IFileHooksReadingProvider {

    String entryName

    AbstractMetaInfInfoReader(String entryName) {
        this.entryName = entryName
    }

    @Override
    void handleFileHooksReading(FileHooksReadingContext context) {
        context.hooksInfo.file.asFileTree().doThenClose { fileTree ->
            fileTree.getEntry(entryName)?.let { hooksInfoEntry ->
                log.info("Found '${entryName}' in ${context.hooksInfo.file.name}, loading bytecode hooks...")
                readMetaInfEntry(context, fileTree, hooksInfoEntry)
            }
        }
    }

    abstract void readMetaInfEntry(FileHooksReadingContext context, IFileTree fileTree, IFileTreeEntry fileTreeEntry)
}
