package io.tswf.minecraft.metamorphosis.core.reading.reader

import com.google.auto.service.AutoService
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.groovy.bettergroovy.io.tree.IFileTree
import io.tswf.groovy.bettergroovy.io.tree.IFileTreeEntry
import io.tswf.minecraft.metamorphosis.core.model.FileHooksInfo
import io.tswf.minecraft.metamorphosis.core.reading.FileHooksReadingContext
import io.tswf.minecraft.metamorphosis.core.reading.IFileHooksReadingProvider
import io.tswf.minecraft.metamorphosis.core.util.constants.ZipEntryNames

@Slf4j
@AutoService(IFileHooksReadingProvider)
@CompileStatic
class JsonHooksInfoReader extends AbstractMetaInfInfoReader {

    Gson gson = new GsonBuilder()
        .tap {
            setPrettyPrinting()
        }
        .create()

    JsonHooksInfoReader() {
        super(ZipEntryNames.HOOKS_JSON)
    }

    @Override
    void readMetaInfEntry(FileHooksReadingContext context, IFileTree fileTree, IFileTreeEntry fileTreeEntry) {
        fileTreeEntry.inputStream.newReader("UTF-8").doThenClose { zipEntryReader ->
            def readedHooksInfo = gson.fromJson(zipEntryReader, FileHooksInfo)
            context.hooksInfo.addAll(readedHooksInfo)
        }
    }
}
