package io.tswf.minecraft.metamorphosis.core.scanning

import groovy.transform.CompileStatic
import groovy.transform.Memoized
import groovy.util.logging.Slf4j
import io.tswf.minecraft.metamorphosis.core.exception.FilesToScanProvidingException
import io.tswf.minecraft.metamorphosis.core.model.FileHooksInfo
import io.tswf.minecraft.metamorphosis.core.reading.FileHooksReader

@Slf4j
@CompileStatic
class HooksInfoScanner {

    static boolean classpathScanEnabled = System.findGlobalProperty("metamorphosis.hooks.classpath.enabled")
        .orElse("null")
        .toLowerCase() == "true"

    @Lazy
    static List<IFilesToScanProvider> filesToScanProviders = ServiceLoader.load(IFilesToScanProvider)
        .toList()

    @Memoized
    static List<FileHooksInfo> scanAllHooks() {
        log.info("Start scanning for bytecode manipulation hooks...")
        def filesToScan = getFilesToScan()

        log.info("Total ${filesToScan.size()} filtes marked to be scanned: ${ filesToScan.map { it.name } }")

        def result = new ArrayList<FileHooksInfo>()

        for (def fileToScan : filesToScan) {
            def fileHooksInfo = FileHooksReader.readFileHooks(fileToScan)
            result << fileHooksInfo
        }

        log.info("Scanning done!")

        return result
    }

    @Memoized
    static List<File> getFilesToScan() {
        def result = new ArrayList<File>()
        for (def filesProvider : filesToScanProviders) {
            try {
                filesProvider.getFilesToScan()?.let {
                    result.addAll(it)
                }
            } catch (Throwable e) {
                throw new FilesToScanProvidingException(e, filesProvider)
            }
        }

        if (classpathScanEnabled) {
            System.getClasspathList().stream()
                .filter { it.protocol == "file" }
                .forEach {
                    result << it.toURI().toFile()
                }
        }

        return result.stream()
            .distinctBy { it.absoluteCanonicalFile.path }
            .toList()
    }
}
