package io.tswf.minecraft.metamorphosis.core.side

import groovy.transform.CompileStatic
import groovy.transform.Memoized

@CompileStatic
class SidesResolver {

    @Lazy
    static List<ICurrentSidesProvider> currentSidesProviders = ServiceLoader.load(ICurrentSidesProvider)
        .toList()

    @Memoized
    static EnumSet<Side> getCurrentSides() {
        def result = EnumSet.noneOf(Side)
        for (def sidesProvider : currentSidesProviders) {
            sidesProvider.getCurrentSides()?.let {
                result.addAll(it)
            }
        }

        return result
    }

    @Memoized
    static boolean isClientPresented() {
        return Side.CLIENT in getCurrentSides()
    }

    @Memoized
    static boolean isServerPresented() {
        return Side.SERVER in getCurrentSides()
    }
}
